dnl grub_ASM_USCORE checks if C symbols get an underscore after
dnl compiling to assembler.
dnl Written by Pavel Roskin. Based on grub_ASM_EXT_C written by
dnl Erich Boleyn and modified by OKUJI Yoshinori
AC_DEFUN(grub_ASM_USCORE,
[AC_REQUIRE([AC_PROG_CC])
AC_MSG_CHECKING([if C symbols get an underscore after compilation])
AC_CACHE_VAL(grub_cv_asm_uscore,
[cat > conftest.c <<\EOF
int
func (int *list)
{
  *list = 0;
  return *list;
}
EOF

if AC_TRY_COMMAND([${CC-cc} ${CFLAGS} -S conftest.c]) && test -s conftest.s; then
  true
else
  AC_MSG_ERROR([${CC-cc} failed to produce assembly code])
fi

if grep _func conftest.s >/dev/null 2>&1; then
  grub_cv_asm_uscore=yes
else
  grub_cv_asm_uscore=no
fi

rm -f conftest*])

if test "x$grub_cv_asm_uscore" = xyes; then
  AC_DEFINE_UNQUOTED([HAVE_ASM_USCORE], $grub_cv_asm_uscore,
    [Define if C symbols get an underscore after compilation])
fi

AC_MSG_RESULT([$grub_cv_asm_uscore])
])

# hurd_SYSDEPS
#
# Find and link the header files appropriate for a given
# configuration.
#
# This function takes two arguments.  The first is a list of paths
# (relative to the ${srcdir}, i.e. the location of the configure
# script the macro is called from) to search for the header files.
#
# The second parameter is a list of header files to find and install.
# Each element may optionally contain a source and destination.  For
# instance, if the file to look for is foo.h, however, it should be
# installed as bar/foo.h, it should be listed as foo.h:bar/foo.h.
#
# For instance, hurd_SYSDEPS([sysdeps/$KERNEL/$ARCH
#                             sysdeps/$KERNEL
#                             sysdeps/generic
#                             sysdeps/$ARCH],
#                            [foo.h:hurd/foo.h
#                             bits/foo.h:hurd/bits/foo.h
#                             bar.h])
AC_DEFUN([hurd_SYSDEPS],
[AC_REQUIRE([AC_PROG_AWK])
 for ac_headers in `echo "$2"`
 do
   ac_SRC=`echo ${ac_headers} \
           | ${AWK} 'BEGIN { FS = ":" } { print $dnl
1; }'`
   ac_DEST=`echo ${ac_headers} \
           | ${AWK} 'BEGIN { FS = ":" } { if (NF == 1) print $dnl
1;  else print $dnl
2 }'`

   ac_header_ok=0
   for ac_dirs in `echo "$1"`
   do
     if test -e "${srcdir}/${ac_dirs}/${ac_SRC}"
     then
       AC_CONFIG_LINKS("include/${ac_DEST}":"${ac_dirs}/${ac_SRC}")
       ac_header_ok=1
       break
     fi
   done
   if test ${ac_header_ok} -ne 1
   then
     AC_MSG_ERROR([The header file ${ac_SRC} is required by, but not provided dnl
for, this configuration.  Report this to the maintainer.])
   fi
 done])
  
