/* RPC magic transformations.
   Copyright (C) 2002 Free Software Foundation, Inc.
   Written by Neal H. Walfield.

   This file is part of the GNU Hurd.

   The GNU Hurd is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   The GNU Hurd is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA. */


/* Input:

     Macro RPC_PREFIX:   Must be defined; prefix for the C stub.
     Macro RPC_POSTFIX:  Must be defined; postfix for the C stub.
     Macro RPC_SEQNO:    If defined, stubs will be called with the
                         sequence number of the rpc.

*/

#include "server-handles.h"
#include <idl4/interface_v4.h>

#undef _RPC_PARAMETERS_SEQNO
#ifdef RPC_SEQNO
#define _RPC_PARAMETERS_SEQNO , msg_seqno_t seqno
#else
#define _RPC_PARAMETERS_SEQNO
#endif

#undef _RPC_ARGS_SEQNO
#ifdef RPC_SEQNO
#define _RPC_ARGS_SEQNO , shi->seqno
#else
#define _RPC_ARGS_SEQNO
#endif

#ifndef __HURD_L4_RPC_MAGIC_H__
#define __HURD_L4_RPC_MAGIC_H__

/* Thread local variables set by the manager thread.  */
#if 0
extern __thread struct server_handle_bucket *manager_bucket;
extern __thread int (*manager_dispatcher) (int (*callback) (void *));
extern __thread msg_id_t manager_current_msg_id;
#else
extern struct server_handle_bucket *manager_bucket;
extern int (*manager_dispatcher) (int (*callback) (void *));
extern msg_id_t manager_current_msg_id;
#endif

extern task_t thread_to_task (L4_ThreadId_t);

/* Generate an rpc stub.  FUNCTION is the rpc name; ARGTYPES the
   parameter list as returned from RPC_TYPES(); and args the argument
   list as returned from RPC_ARGS().  Makes the following assumptions:
   the rpc returns an error_t and the first argument is a
   user_handle_id_t on the wire and a struct server_handle_info * in
   the stub.  */
#define RPC_GEN(function, argtypes, args) _RPC_GEN(function, argtypes, args)
#define _RPC_GEN(function, argtypes, realstub_argtypes, args) \
  extern error_t RPCNAME(function) realstub_argtypes; \
  extern inline error_t \
  RPCNAME(_frob_ ## function) argtypes \
  { \
    error_t err; \
    struct rpc_info info; \
    server_handle_t sh \
      = { uhid, thread_to_task (* (L4_ThreadId_t *) &caller) }; \
    struct server_handle_info *shi \
      = server_handle_lookup (manager_bucket, sh, 0); \
    \
    int invoke (void *x) \
      { \
        return RPCNAME (function) args; \
      } \
    \
    if (! shi) \
      return EOPNOTSUPP; \
    \
    err = server_handle_begin_rpc (shi, sh, manager_current_msg_id, \
                                   &info); \
    if (! err) \
      { \
        if (manager_dispatcher) \
          err = manager_dispatcher (invoke); \
        else \
          err = invoke (0); \
        \
	server_handle_end_rpc (shi, &info); \
      } \
    server_handle_deref (shi); \
    return err; \
  } \
  \
  IDL4_PUBLISH_ ## function (RPCNAME(_frob_ ## function));

/* Return the C name for the rpc NAME by prepending RPC_PREFIX and
   appending RPC_POSTFIX.  */
#define RPCNAME(name) _RPCNAME1(RPC_PREFIX, name, RPC_POSTFIX)
#define _RPCNAME1(a, b, c) _RPCNAME2(a, b, c)
#define _RPCNAME2(a, b, c) a ## b ## c

/* Massage the parameter list.  */
#define RPC_PARAMETERS(argtypes...) \
  (CORBA_Object caller, \
   const user_handle_id_t uhid , ## argtypes, \
   idl4_server_environment *env) , \
  (CORBA_Object caller, \
   struct server_handle_info *shi _RPC_PARAMETERS_SEQNO , ## argtypes, \
   idl4_server_environment *env)

/* Massages the argument list.  Must parallel the TYPES function.  */
#define RPC_ARGS(args...) (caller, shi _RPC_ARGS_SEQNO , ## args, env)

#endif /*  __HURD_L4_RPC_MAGIC_H__ */
