/* Started at the University of Dresden then modified at University of
   Karlsruhe.  Could not find a copyright notice. */

/* Copyright (C) 2002 Free Software Foundation, Inc.

   This file is part of the GNU Hurd.
   
   The GNU Hurd is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.
   
   The GNU Hurd is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with the GNU Hurd; see the file COPYING.  If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139,
   USA.  */


#include <stdio.h>
#include <string.h>
#include <machine/vmparam.h>
#include <l4/kernel_configuration_page.h>

#include "assert.h"
#include "multiboot.h"
#include "elf-load.h"
#include "die.h"
#include "panic.h"
#include "serial-init.h"
#include "boot-modules.h"

#define STRINGIFY(x) STRINGIFY_1(x)
#define STRINGIFY_1(x) #x

#define L4_MODULE (0)
#define SIGMA0_MODULE (1)
#define ROOT_SERVER_MODULE (2)
#define FIRST_BOOT_MODULE (3)

/* Amount of physical memory.  */
L4_Word_t physical_memory;
L4_Word_t physical_memory_lower;

extern L4_Word_t _start, _end, _etext;

/* Statistics on system modules.  */
struct system_module
{
  L4_Word_t image_start;
  L4_Word_t image_end;
  L4_Word_t start;
  L4_Word_t end;
  L4_Word_t entry;
  char commandline[1024];
};

static struct system_module system_modules[FIRST_BOOT_MODULE];

/* Cookie in exec_read_exec.  */
struct loader_cookie
{
  struct system_module *module;
  int module_index;
  int used;	   /* Must be initialized to 0.  */
};

/* User modules.  */
static char boot_modules_buffer[4096 * 8];
static struct boot_modules_header *boot_modules_header
  = (void *) boot_modules_buffer;

#ifdef DEBUG
#define dump_mbi(mbi) \
	  printf ("multiboot_info (%p) at %s@%d:\n" \
		  "  flags = %x\n" \
		  "  mem_lower = %d\n" \
		  "  mem_upper = %d\n" \
		  "  boot_device = %x\n" \
		  "  cmdline = %s\n" \
		  "  mods_count = %d\n" \
		  "  mods_addr = %p\n", \
		  mbi, \
		  __FUNCTION__, \
		  __LINE__, \
		  mbi->flags, \
		  mbi->mem_lower, \
		  mbi->mem_upper, \
		  mbi->boot_device, \
		  (char *) mbi->cmdline, \
		  mbi->mods_count, \
		  mbi->mods_addr)

#define dump_boot_module(m) \
      printf ("boot_module:\n" \
	      "  have_range: %d\n" \
	      "  have_commandline: %d\n" \
	      "  padding: %d\n" \
	      "  size: %d\n" \
	      "  start: %p\n" \
	      "  end: %p\n" \
	      "  commandline_length: %d\n" \
	      "  commandline_offset: %d\n" \
	      "  commandline: %s\n", \
	      (m)->flags.have_range, \
	      (m)->flags.have_commandline, \
	      (m)->flags.padding, \
	      (m)->size, \
	      (m)->start, \
	      (m)->end, \
	      (m)->commandline_length, \
	      (m)->commandline_offset, \
	      (m)->commandline);

#define dump_system_module(m) \
	printf ("system_module:\n" \
		"  image_start: %x\n" \
		"  image_end: %x\n" \
		"  start: %x\n" \
		"  end: %x\n" \
		"  entry: %x\n" \
		"  commandline: %s\n", \
		(m)->image_start, \
		(m)->image_end, \
		(m)->start, \
		(m)->end, \
		(m)->entry, \
		(m)->commandline);

#define short_pause() \
	do \
	  { \
	    int i; \
	    for (i = 0; i < 1000000; ) \
	      (volatile int) i = (volatile int) i + 1; \
	  } \
	while (0)

#define dump_kcp(k) \
	do \
	  { \
	    int i; \
	    printf ("KCP:\n" \
	            "  sigma0.sp = %p\n" \
		    "  sigma0.ip = %p\n" \
		    "  sigma0.low = %p\n" \
		    "  sigma0.high = %p\n" \
		    "  sigma1.sp = %p\n" \
		    "  sigma1.ip = %p\n" \
		    "  sigma1.low = %p\n" \
		    "  sigma1.high = %p\n", \
		    kcp->sigma0.sp, kcp->sigma0.ip, \
		    kcp->sigma0.low, kcp->sigma0.high, \
		    kcp->sigma1.sp, kcp->sigma1.ip, \
		    kcp->sigma1.low, kcp->sigma1.high, \
		    kcp->root_server.sp, kcp->root_server.ip); \
	    \
	    short_pause(); \
	    \
	    printf ("  root_server.sp = %p\n" \
		    "  root_server.ip = %p\n", \
                    "  root_server.low = %p\n" \
		    "  root_server.high = %p\n" \
		    "  kdebug.config1 = %p\n" \
		    "  kdebug_config2 = %p\n" \
		    "  MainMem.low = %p\n" \
		    "  MainMem.high = %p\n", \
		    kcp->root_server.low, kcp->root_server.high, \
		    kcp->Kdebug_config[0], kcp->Kdebug_config[1], \
		    kcp->MainMem.low, kcp->MainMem.high); \
	    \
	    short_pause(); \
	    \
	    for (i = 0; i < 5; i ++) \
              printf ("  DedicatedMem[%d].low = %p\n" \
                      "  DedicatedMem[%d].high = %p\n", \
                      i, kcp->DedicatedMem[i].low, \
	              i, kcp->DedicatedMem[i].high); \
         } \
       while (0)

#else
#define dump_mbi(mbi) do { } while (0)
#define dump_boot_module(m) do { } while (0)
#define dump_system_module(m) do { } while (0)
#define dump_kcp(m) do { } while (0)
#endif /* DEBUG */

/* Read in the meta data of the executable.  */
int
exec_read (void *handle, vm_offset_t file_ofs,
	   void *buf, vm_size_t size,
	   vm_size_t *out_actual)
{
  struct loader_cookie *info = handle;
  int i;
 
  if (! (buf >= (void *) &_etext && buf + size <= (void *) &_end))
    panic ("Bad memory access by elf decoder.");

  memcpy (buf, (void *) info->module->image_start + file_ofs, size);

  *out_actual = size;
  return 0;
}

/* "Read" in parts of the executable.  As we map virtual to physical
   memory one-to-one, this is trivial implement, however, limited in
   functionality: any overlap kills us as we cannot remap.  */
static int
exec_read_exec (void *handle,
	        vm_offset_t file_ofs, vm_size_t file_size,
	        vm_offset_t mem_addr, vm_size_t mem_size,
	        exec_sectype_t section_type)
{
  int
    overlaps_range (L4_Word_t tst_begin, L4_Word_t tst_end, 
		    L4_Word_t mem_begin, L4_Word_t mem_end)
    {
      return ! ((tst_begin <= mem_begin && tst_end <= mem_begin)
		|| (tst_begin >= mem_end  && tst_end >= mem_end));
    }

  int i;
  struct loader_cookie *info = handle;
  vm_offset_t start = mem_addr;
  vm_offset_t end = mem_addr + mem_size;
  struct boot_module *m;

  assert (mem_size > 0);
  assert (start < end);

  /* We do not need debugging sections.  */
  if (! (section_type & (EXEC_SECTYPE_ALLOC)))
    return 0;

  if (! info->used)
    {
      info->used = 1;
      info->module->start = ~0;
      info->module->end = 0;
    }

  if (start < info->module->start)
    info->module->start = start;
  if (info->module->end < end)
    info->module->end = end;

  if (end >= physical_memory)
    panic ("Module's elf section out of range.\n"
	   "\tsection: %08x-%08x\n\tend of physical memory: %08x",
	   start, end, physical_memory);
  
  /* Check that we do not overwrite ourself or the modules.  */
  if (overlaps_range (start, end,
		      (L4_Word_t) &_start, (L4_Word_t) &_end))
    panic ("Extracting data from module would overwrite loader.\n"
	   "\tLoader: %08x-%08x\n\tModule section: %08x-%08x",
	   &_start, &_end, start, end);

  for (i = 0; i < info->module_index; i ++)
    if (overlaps_range (start, end,
			system_modules[i].start,
			system_modules[i].end))
      panic ("Extracting section %08x-%08x from module %d would\n"
	     "overwrite data extracted from module %d (%08x-%08x).",
	     start, end, info->module_index, i,
	     system_modules[i].start,
	     system_modules[i].end);

  for (i = info->module_index,
	 m = (void *) boot_modules_header
	     + boot_modules_header->modules_offset;
       i < boot_modules_header->module_count;
       i ++, m = (void *) m + m->size)
    if (overlaps_range (start, end, m->start, m->end))
      panic ("Extracting section %08x-%08x from module %d would\n"
	     "overwrite data in module %d (%08x-%08x).",
	     start, end, info->module_index, i,
	     m->start, m->end);

  /* Copy the data.  */
  memcpy ((void *) start, (void *) info->module->image_start + file_ofs,
	  file_size);

  /* Zero the rest.  */
  if (file_size < mem_size)
    memset ((void *) start + file_size, 0, mem_size - file_size);

  return 0;
}

/* Decode and load module MODULE_NUMBER into memory.  */
void
load_module (int module_number)
{
  int err;
  struct system_module *module;
  struct loader_cookie loader_cookie;
  exec_info_t exec_info;

  assert (module_number < FIRST_BOOT_MODULE);

  module = &system_modules[module_number];
  if (module->commandline)
    printf ("Loading %s...", module->commandline);
  else
    printf ("Loading module %d...", module_number);

  loader_cookie.module = module;
  loader_cookie.module_index = module_number;
  loader_cookie.used = 0;

  err = exec_load (exec_read, exec_read_exec, &loader_cookie,
		   &exec_info);
  if (err)
    panic (" failed: %s", ex_error_message (err));
  else
    printf (" ok.\n");

  module->entry = exec_info.entry;

  dump_system_module (module);
}

/* Bring the system up by patching the kernel configuration page and
   starting L4, sigma0 and the root task.  */
int
main (unsigned int magic, struct multiboot_info *mbi)
{
  int err;

  char *option;
  int i;

  L4_KernelConfigurationPage_t *kcp;

  static char sigma0_stack[64];
  static char root_server_stack[64];

  serial_init (0x3f8, 115200);

  printf ("LADEN\n");

  dump_mbi (mbi);

  /* Assert that we have been "multiboot-booted."  */
  if (magic != MULTIBOOT_BOOTLOADER_MAGIC)
    panic ("No multiboot magic; got %x, not %x\n",
	   magic, MULTIBOOT_BOOTLOADER_MAGIC);

  /* We need at least three boot modules, the L4 kernel, sigma0 and
     the root task.  */
  if (! (mbi->flags & (1 << 3)))
    panic ("No modules found; nothing to load.");

  if (mbi->mods_count < FIRST_BOOT_MODULE)
    panic ("Only found %d modules, need at least three (L4, sigma0 "
	   "and the root task)", mbi->mods_count);

  /* We require that the memory size be passed (cf crt0).  */
  if (! (mbi->flags & 1))
    panic ("Memory configuaration not pass by boot loader.");

  /* And the command line.  */
  if (! (mbi->flags & (1 << 2)))
    {
      printf ("Warning: no kernel command line.\n");
      mbi->cmdline = (unsigned long) "";
    }

  /* Construct the module data structure.  */
  for (i = 0; i < FIRST_BOOT_MODULE; i ++)
    {
      int len;
      struct multiboot_module *modules = (void *) mbi->mods_addr;

      system_modules[i].image_start = modules[i].mod_start;
      system_modules[i].image_end = modules[i].mod_end;

      len = strlen ((char *) modules[i].string);
      if (len >= sizeof (system_modules[i].commandline))
	printf ("Warning: truncating module %d's command line.\n", i);
      if (len > 0)
	memcpy (system_modules[i].commandline,
		(void *) modules[i].string, len);
      system_modules[i].commandline[len] = 0;
    }

  /* Fill the user modules' data structure.  */
  {
    struct multiboot_module *modules = (void *) mbi->mods_addr;
    int len;
    struct boot_module *m;

    /* Magic!  */
    *(L4_Word_t *) &boot_modules_header->magic = BOOT_MODULE_MAGIC;
    boot_modules_header->flags.padding = 0;

    /* Command line.  */
    len = strlen ((char *) modules[ROOT_SERVER_MODULE].string);
    if (len > 0)
      {
	boot_modules_header->flags.have_commandline = 1;
	boot_modules_header->commandline_length = len;
	boot_modules_header->commandline_offset
	  = (int) (& ((struct boot_modules_header *) 0)->commandline);
	memcpy (boot_modules_header->commandline,
		(void *) modules[ROOT_SERVER_MODULE].string, len);

	m = (void *) boot_modules_header + sizeof (struct boot_modules_header)
	  + ((len + (sizeof (L4_Word_t) - 1)) & ~(sizeof (L4_Word_t) - 1));
      }
    else
      {
	boot_modules_header->flags.have_commandline = 0;
	m = (void *) boot_modules_header
	  + sizeof (struct boot_modules_header);
      }

    /* Other modules.  */
    boot_modules_header->flags.have_modules = 1;

    boot_modules_header->size = sizeof (boot_modules_header);
    boot_modules_header->module_count = 0;
    boot_modules_header->modules_offset
      = (void *) m - (void *) boot_modules_header;

    for (i = FIRST_BOOT_MODULE; i < mbi->mods_count;
	 i ++)
      {
	m->flags.have_commandline = 1;
	m->flags.padding = 0;

	m->start = modules[i].mod_start;
	m->end = modules[i].mod_end;

	m->commandline_length = strlen ((char *) modules[i].string);
	m->commandline_offset
	  = (int) (&*((struct boot_module *)0)->commandline);
	memcpy (m->commandline, (char *) modules[i].string,
		m->commandline_length);

	m->size = ((sizeof (struct boot_module)
		    + m->commandline_length + (sizeof (L4_Word_t) - 1))
		   & ~(sizeof (L4_Word_t) - 1));
	boot_modules_header->size += m->size;
	m = (void *) m + m->size;

	boot_modules_header->module_count ++;
      }
  }

#if 0
  option = strstr ((char *) mbi->cmdline, "-maxmem");
  if (option)
    {
      mbi->mem_upper =
	1024 * strtoul (option + strlen("-maxmem") + 1,
			NULL, 10) - 1;

      printf ("Limiting memory to %d MB (%d KB)\n",
	      mbi->mem_upper / 1024 + 1,
	      mbi->mem_upper);
    }
#endif

  physical_memory = 1024 * (mbi->mem_upper + mbi->mem_lower);
  physical_memory_lower = 1024 * mbi->mem_lower;
  printf ("%d Kb of physical memory.\n", physical_memory / 1024);

  load_module (L4_MODULE);

  /* Figure out which L4 kernel we are booting.  */
  printf ("Frobbing kernel...");
  {
    char *p;
    int known = 0;

    for (p = (char *) system_modules[L4_MODULE].start;
	 p < (char *) system_modules[L4_MODULE].end;
	 p ++)
      if (memcmp (p, "L4Ka/Pistachio", 14) == 0)
	{
	  /* it's the Karlsruhe version */
	  printf (" found pistachio.\n");
	  known = 1;
	  break;
	}

    if (! known)
      panic ("Unrecognized kernel.");
  }

  /* Find the kernel configuration page.  */
  printf ("Searching for the kernel configuration page...");
  {
    L4_KernelConfigurationPage_t *k = 0;

    for (k = (void *) (system_modules[L4_MODULE].start & PAGE_MASK);
	 (void *) k < (void *) system_modules[L4_MODULE].end;
	 k = (void *) k + PAGE_SIZE)
      {
	if (L4_MAGIC == k->magic)
	  {
	    if (kcp)
	      panic (" failed.\n"
		     "Found a second kernel configuration page at %p!",
		     k);

	    kcp = k;
	  }
      }

    if (! kcp)
      panic ("Did not find kernel configuration page magic.");
  }
  printf (" ok.\n", kcp);


  /* Load sigma0.  */
  load_module (SIGMA0_MODULE);

  kcp->sigma0.low = system_modules[SIGMA0_MODULE].start;
  kcp->sigma0.high = ((system_modules[SIGMA0_MODULE].end
		      + PAGE_SIZE - 1) & PAGE_MASK) - 1;
  kcp->sigma0.ip = system_modules[SIGMA0_MODULE].entry;
  kcp->sigma0.sp = (L4_Word_t) (sigma0_stack
				+ sizeof (sigma0_stack));

  /* Load the root server.  */
  load_module (ROOT_SERVER_MODULE);

  kcp->root_server.low = system_modules[ROOT_SERVER_MODULE].start;
  kcp->root_server.high = ((system_modules[ROOT_SERVER_MODULE].end
			   + PAGE_SIZE - 1) & PAGE_MASK) - 1;
  kcp->root_server.ip = system_modules[ROOT_SERVER_MODULE].entry;
  kcp->root_server.sp = (L4_Word_t) (root_server_stack
				     + sizeof (root_server_stack));

  /* Clear sigma1 -- we do not use it.  */
  kcp->sigma1.low = 0;
  kcp->sigma1.high = 0;
  kcp->sigma1.ip = 0;
  kcp->sigma1.sp = 0;

  /* Fill in the memory layout.  */
  kcp->MainMem.low = 0;
  kcp->MainMem.high = physical_memory;

  i = 0;

  /* XXX: x86 hack.  We must reserve 640k - 1M.  */
  kcp->DedicatedMem[i].low = physical_memory_lower;
  kcp->DedicatedMem[i].high = 1024 * 1024;
  i ++;

  /* Boot modules header.  */
  kcp->DedicatedMem[i].low = (L4_Word_t) boot_modules_buffer;
  kcp->DedicatedMem[i].high
    = kcp->DedicatedMem[i].low + boot_modules_header->size;
  i ++;

  /* User modules--needed by the root task.  */
  if (boot_modules_header->module_count > 0)
    {
      L4_Word_t start, end;
      struct boot_module *m
	= (void *) boot_modules_header + boot_modules_header->modules_offset;
      int j;
      
      start = m->start;
      end = m->end;

      dump_boot_module (m);

      for (j = 1, m = (void *) m + m->size;
	   j < boot_modules_header->module_count;
	   j ++, m = (void *) m + m->size)
	{
	  if (m->start < start)
	    start = m->start;
	  if (m->end > end)
	    end = m->end;

	  dump_boot_module (m);
	}

      kcp->DedicatedMem[i].low = start;
      kcp->DedicatedMem[i].high = end - 1;
      i ++;
    }

  for (; i < sizeof (kcp->DedicatedMem) / sizeof (kcp->DedicatedMem[0]);
       i ++)
    {
      kcp->DedicatedMem[i].low = 0;
      kcp->DedicatedMem[i].high = 0;
    }

  kcp->BootInfo = (L4_Word_t) boot_modules_header;

  dump_kcp (kcp);

  printf ("Booting L4.\n");

  {
    typedef void (*l4_main_t) (void);
    l4_main_t f = (void *) system_modules[L4_MODULE].entry;
    f ();
  }

  panic ("Aieeee! L4 failed to boot!");

  for (;;)
    ;
}
