/*********************************************************************
 *                
 * Copyright (C) 2001, 2002,  Karlsruhe University
 *                
 * File path:     l4/x86/types.h
 * Description:   x86 specific type declararions
 *                
 * @LICENSE@
 *                
 ********************************************************************/


#include "serial-init.h"
#include <l4/arch/types.h>

/**
 * out_u8:      outputs a byte on an io port
 * @port:       port number
 * @val:        value
 */
inline void
out_u8(const L4_Int16 port, const L4_Int8 val)
{
    /* GCC can optimize here if constant */
    if (__builtin_constant_p(port) && (port < 0x100))
        __asm__ __volatile__("outb      %1, %0\n"
                             :
                             : "dN"(port), "a"(val));
    else
        __asm__ __volatile__("outb      %1, %%dx\n"
                             :
                             : "d"(port), "a"(val));
}

/**
 * in_u8:       reads a byte from a port
 * @port:       port number
 *
 * Returns the read byte.
 */
inline L4_Int8
in_u8 (const L4_Int16 port)
{
    L4_Int8 tmp;
    /* GCC can optimize here if constant */
    if (__builtin_constant_p(port) && (port < 0x100))
        __asm__ __volatile__("inb %1, %0\n"
                             : "=a"(tmp)
                             : "dN"(port));
    else
        __asm__ __volatile__("inb %%dx, %0\n"
                             : "=a"(tmp)
                             : "d"(port));
    return tmp;
}

/* Typically 3F8 for com0 and 115200 to be compatible with the kernel
   debugger.  */
void
serial_init (int port, int speed)
{
  volatile int i;
#define IER     (port+1)
#define EIR     (port+2)
#define LCR     (port+3)
#define MCR     (port+4)
#define LSR     (port+5)
#define MSR     (port+6)
#define DLLO    (port+0)
#define DLHI    (port+1)

    out_u8(LCR, 0x80);          /* select bank 1        */
    for (i = 10000000; i--; );
    out_u8(DLLO, (((115200/speed) >> 0) & 0x00FF));
    out_u8(DLHI, (((115200/speed) >> 8) & 0x00FF));
    out_u8(LCR, 0x03);          /* set 8,N,1            */
    out_u8(IER, 0x00);          /* disable interrupts   */
    out_u8(EIR, 0x07);          /* enable FIFOs */
    out_u8(IER, 0x01);          /* enable RX interrupts */
    in_u8(IER);
    in_u8(EIR);
    in_u8(LCR);
    in_u8(MCR);
    in_u8(LSR);
    in_u8(MSR);
}


