#ifndef __STRING_H__
#define __STRING_H__

#include <stddef.h>
#include <sys/malloc.h>

static inline int
strcmp (const char *s1, const char *s2)
{
  for (; *s1 && *s2; s1 ++, s2 ++)
    if (*s1 < *s2)
      return -1;
    else if (*s1 > *s2)
      return 1;

  /* Either *s1 is nul, *s2 is nul or they both are.  */
  if (*s1 == *s2)
    return 0;
  if (*s1)
    return -1;
  else
    return 1;
}

static inline int
memcmp (const void *a, const void *b, size_t s)
{
  const char *x = a, *y = b;

  for (; s > 0; x ++, y ++, s --)
    if (*x < *y)
      return -1;
    else if (*x > *y)
      return 1;

  return 0;
}
  

static inline int
strlen (const char *s)
{
  int i = 0;
  
  for (; *s; s ++, i ++)
    ;

  return i;
}

static inline char *
strchr (const char *s, int c)
{
  for (; *s; s ++)
    if (*s == c)
      return (char *) s;

  return 0;
}

static inline char *
strstr (const char *haystack, const char *needle)
{
  int hlen = strlen (haystack);
  int nlen = strlen (needle);

  if (nlen == 0)
    return (char *) haystack;

  for (; nlen <= hlen; haystack ++, hlen --)
    if (memcmp (haystack, needle, nlen) == 0)
      return (char *) haystack;

  return 0;
}

static inline void *
memset (void *block, int c, size_t size)
{
  char *b = block;
  
  for (; size > 0; b ++, size --)
    *b = (unsigned char) c;

  return b;
}

static inline void
bzero (void *block, size_t size)
{
  memset (block, 0, size);
}

static inline void *
memcpy (void *dest, const void *src, size_t size)
{
  char *d = dest;
  const char *s = src;

  for (; size > 0; d ++, s ++, size --)
    *d = *s;

  return dest;
}

static inline void
bcopy (const void *src, void *dest, size_t size)
{
  memcpy (dest, src, size);
}

static inline char *
strdup (const char *s)
{
  int len = strlen (s) + 1;
  char *r = malloc (len);

  if (r)
    memcpy (r, s, len);

  return r;
}

#endif
