#ifndef __DLMALLOCPARAMS_H__
#define __DLMALLOCPARAMS_H__

#include <l4/l4.h>

#include <machine/vmparam.h>
#include <sys/dlmallocglue.h>

/* We fake mmap.  */
#define HAVE_MMAP 1
/* Nor do we have mremap, however, we do not try to emulate it.  */
#define HAVE_MREMAP 0
/* Let malloc clear the memory for us.  */
#define MMAP_CLEARS 0

/* Our size_t.  */
#define INTERNAL_SIZE_T L4_Word_t

/* Aggressively use mmap.  */
#define DLMALLOC_MMAP_OVERHEAD (3 * sizeof (INTERNAL_SIZE_T))
#define DEFAULT_MMAP_THRESHOLD \
	((dlmalloc_chunk_threshold) - (DLMALLOC_MMAP_OVERHEAD))

/* If morecore fails (which it always will), mmap is used.  By
   default, malloc requests a 1M region!  This will typically be
   dramatically reduced as we are working with physical memory.  */
#define MMAP_AS_MORECORE_SIZE (dlmalloc_arena_size)

/* We want locking.  The user must define dlmalloc_{un,}lock.  These
   must return 0 on success; otherwise, the operation is canceled.  */
#define USE_PUBLIC_MALLOC_WRAPPERS
#define MALLOC_PREACTION dlmalloc_lock()
#define MALLOC_POSTACTION dlmalloc_unlock()

/* We have none of these!  */
#define LACKS_UNISTD_H
#define LACKS_SYS_PARAM_H
#define LACKS_SYS_MMAN_H
#define LACKS_FCNTL_H

/* Use malloc's inline versions.  */
#define USE_MEMCPY 0

/* If this is not defined, malloc tries to open /dev/zero and map from
   there.  */
#define MAP_ANONYMOUS 0

/* We do not have sbrk and zalloc is much better anyway.  Thus, we
   always fail.  */
#define MORECORE(x) MORECORE_FAILURE

/* If malloc fails, panic.  */
#define MALLOC_FAILURE_ACTION (dlmalloc_failed ())

/* Again, we fake mmap: we provide the implementation details that
   malloc expects and a few defines, however, the only relevant data
   that malloc provides is the size argument.  Thus, we use dummy
   values for the rest.  */
#define PROT_READ (0)
#define PROT_WRITE (0)
#define MAP_PRIVATE (0)

/* Declare them.  */
int dlmalloc_extant;
int dlmalloc_alloc;
int dlmalloc_free;

/*
  void *
  mmap (void *addr, L4_Word_t size, int protect, int flags,
        int filedes, L4_Word_t offset)
*/

#define mmap(addr, size, protect, flags, filedes, offset)                \
	({                                                               \
	  void *p;                                                       \
	                                                                 \
	  assert ((size) & (PAGE_SIZE - 1));                             \
                                                                         \
	  dlmalloc_extant += (size);                                     \
	  dlmalloc_alloc += (size);                                      \
	                                                                 \
	  p = dlmalloc_allocate ((size));                                \
	  if (! p)                                                       \
	    p = (void *) -1;                                             \
	  else                                                           \
	    {                                                            \
	      /* We need to clear the header's--malloc makes certain     \
		 assumptions. *sigh*  */                                 \
	      ((L4_Word_t *)p)[0] = 0;                                   \
	      ((L4_Word_t *)p)[1] = 0;                                   \
	    }                                                            \
									 \
	  p;								 \
	})

/* int
   munmap (void *addr, L4_Word_t size)
*/
#define munmap(addr, size)                                               \
	({								 \
	  if ((size) & (PAGE_SIZE - 1))					 \
	    {								 \
	      printf ("%s: unmapping %x but size %d(0x%x) is not a "	 \
		      "multiple of page size.",				 \
		      __FUNCTION__, (addr), (size), (size));		 \
	      enter_kdebug ("munmap");					 \
	    }                                                            \
									 \
	  dlmalloc_extant -= (size);					 \
	  dlmalloc_free += (size);					 \
									 \
	  dlmalloc_deallocate ((addr), (size));				 \
	  0;		                                                 \
	})

#endif /* __DLMALLOCPARAMS_H__ */
