/* Iterate a function over the server_handle in a bucket.
   Copyright (C) 1995,99, 2002 Free Software Foundation, Inc.
   Written by Michael I. Bushnell.

   This file is part of the GNU Hurd.

   The GNU Hurd is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   The GNU Hurd is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA. */

#include "server-handles.h"

#include <hurd/ihash.h>

error_t
_server_handle_bucket_class_iterate (struct server_handle_bucket *bucket,
				     struct server_handle_class *class,
				     error_t (*fun)(void *),
				     int have_server_handle_lock)
{
  /* This is obsecenely ineffecient.  ihash and server_handle need to
     cooperate more closely to do it effeciently. */
  struct item
    {
      struct item *next;
      void *p;
    } *list = 0;
  struct item *i, *nxt;
  error_t err;

  error_t enqueue (void *arg)
    {
      struct server_handle_info *const sh = arg;
      struct item *j;

      if (class == 0 || sh->class == class)
	{
	  j = malloc (sizeof (struct item));
	  assert (j);
	  j->next = list;
	  j->p = sh;
	  list = j;
	  sh->refcnt++;
	}
      return 0;
    }

  if (! have_server_handle_lock)
    pthread_mutex_lock (&_server_handle_lock);

  ihash_iterate (bucket->htable, enqueue);

  if (! have_server_handle_lock)
    pthread_mutex_unlock (&_server_handle_lock);

  err = 0;
  for (i = list; i; i = nxt)
    {
      if (!err)
	err = (*fun)(i->p);
      server_handle_deref (i->p);
      nxt = i->next;
      free (i);
    }
  return err;
}

error_t
server_handle_bucket_iterate (struct server_handle_bucket *bucket,
			       error_t (*fun)(void *))
{
  return _server_handle_bucket_class_iterate (bucket, 0, fun, 0);
}
