/* 
   Copyright (C) 1995,96, 2002 Free Software Foundation, Inc.
   Written by Michael I. Bushnell.

   This file is part of the GNU Hurd.

   The GNU Hurd is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   The GNU Hurd is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "server-handles.h"
#include "sh-internal.h"
#include <assert.h>
#include <pthread.h>
#include <hurd/ihash.h>

/* _server_handle_lock must be held on entry.  */
void
_server_handle_complete_deallocate (struct server_handle_info *sh)
{
  assert ((sh->flags & SERVER_HANDLE_HAS_USERS) == 0);

  _server_handle_info_fini (sh);

  *sh->prevp = sh->next;
  if (sh->next)
    sh->next->prevp = sh->prevp;

  sh->bucket->count--;
  sh->class->count--;

  ihash_locp_remove (sh->bucket->htable, sh->hentry);

  pthread_mutex_unlock (&_server_handle_lock);

  if (sh->class->clean_routine)
    (*sh->class->clean_routine)(sh);
  
  free (sh);
}

