/* Create a server_handle bucket
   Copyright (C) 1995,97, 2001,02 Free Software Foundation, Inc.
   Written by Michael I. Bushnell.

   This file is part of the GNU Hurd.

   The GNU Hurd is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   The GNU Hurd is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "server-handles.h"
#include "sh-internal.h"

#include <errno.h>
#include <stdlib.h>
#include <pthread.h>
#include <hurd/ihash.h>

struct server_handle_bucket *
server_handle_create_bucket ()
{
  struct server_handle_bucket *ret;
  error_t err;

  ret = malloc (sizeof (struct server_handle_bucket));
  if (! ret)
    {
      err = ENOMEM;
      goto lose;
    }

  ret->rpcs = ret->flags = ret->count = 0;

  err = ihash_create (&ret->htable);
  if (err)
    goto lose_with_ret;

  err = _server_bucket_init (ret);
  if (err)
    goto lose_with_hash;

  pthread_mutex_lock (&_server_handle_lock);
  ret->next = _server_handle_all_buckets;
  _server_handle_all_buckets = ret;
  pthread_mutex_unlock (&_server_handle_lock);

  return ret;

 lose_with_hash:
  ihash_free (ret->htable);
 lose_with_ret:
  free (ret);
 lose:
  errno = err;
  return NULL;
}
