/* 
   Copyright (C) 1995, 2001,02 Free Software Foundation, Inc.
   Written by Michael I. Bushnell.

   This file is part of the GNU Hurd.

   The GNU Hurd is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   The GNU Hurd is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "server-handles.h"
#include <pthread.h>
#include <assert.h>

void
server_handle_server_handle_deref (void *server_handle_struct)
{
  struct server_handle_info *sh = server_handle_struct;
  int trieddroppingweakrefs = 0;

 retry:
  
  pthread_mutex_lock (&_server_handle_lock);
  
  if (sh->refcnt == 1 && sh->weakrefcnt
      && sh->class->dropweak_routine && !trieddroppingweakrefs)
    {
      pthread_mutex_unlock (&_server_handle_lock);
      (*sh->class->dropweak_routine) (sh);
      trieddroppingweakrefs = 1;
      goto retry;
    }
  
  assert (sh->refcnt);

  sh->refcnt--;
  if (sh->refcnt == 0 && sh->weakrefcnt == 0)
    _server_handle_complete_deallocate (sh);
  else
    pthread_mutex_unlock (&_server_handle_lock);
}

      
