/*
   Copyright (C) 1995,96, 2000,02 Free Software Foundation, Inc.
   Written by Michael I. Bushnell.

   This file is part of the GNU Hurd.

   The GNU Hurd is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   The GNU Hurd is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "server-handles.h"
#include <hurd.h>
#include <pthread.h>

error_t
server_handle_inhibit_class_rpcs (struct server_handle_class *class)
{
  error_t err = 0;

  pthread_mutex_lock (&_server_handle_lock);

  if (class->flags & (SERVER_HANDLE_CLASS_INHIBITED
		      | SERVER_HANDLE_CLASS_INHIBIT_WAIT))
    err = EBUSY;
  else
    {
      struct server_handle_info *sh;
      struct rpc_info *rpc;
      int this_one = 0;

      for (sh = class->server_handle; sh; sh = sh->next)
	for (rpc = sh->current_rpcs; rpc; rpc = rpc->next)
	  {
	    /* Avoid cancelling the calling thread.  */
	    if (pthread_equal (rpc->thread, pthread_self ()))
	      this_one = 1;
	    else
	      pthread_cancel (rpc->thread);
	  }

      pthread_cleanup_push ((void (*) (void *)) pthread_mutex_unlock,
			    (void *) &_server_handle_lock);

      while (class->rpcs > this_one)
	{
	  class->flags |= SERVER_HANDLE_CLASS_INHIBIT_WAIT;
	  pthread_cond_wait (&_server_handle_block,
			     &_server_handle_lock);
	}

      pthread_cleanup_pop (0);

      class->flags &= ~SERVER_HANDLE_CLASS_INHIBIT_WAIT;
      if (! err)
	class->flags |= SERVER_HANDLE_CLASS_INHIBITED;
    }

  pthread_mutex_unlock (& _server_handle_lock);

  return err;
}
