/* Handle interruping rpcs because of notification

   Copyright (C) 1995, 2001,02 Free Software Foundation, Inc.

   Written by Miles Bader <miles@gnu.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "server-handles.h"

/* A linked list of server_handle for which notification has been
   requested.  */
struct server_handle_notify *_server_handle_notifications;

/* Free lists for notify structures.  */
struct server_handle_notify *_server_handle_free_server_handle_notifies;
struct rpc_notify *_server_handle_free_rpc_notifies;

/* Interrupt any rpcs on OBJECT that have requested such.  */
void
server_handle_interrupt_notified_rpcs (void *object,
					server_handle_t server_handle,
					msg_id_t what)
{
  if (_server_handle_notifications)
    {
      struct server_handle_notify *np;

      pthread_mutex_lock (&_server_handle_lock);
      for (np = _server_handle_notifications; np; np = np->next)
	if (np->server_handle == server_handle && np->what == what)
	  {
	    struct rpc_notify *req;
	    for (req = np->reqs; req; req = req->next_req)
	      if (req->pending)
		{
		  req->pending--;
		  pthread_cancel (req->rpc->thread);
		}
	    break;
	  }
      pthread_mutex_unlock (&_server_handle_lock);
    }
}

static void
remove_req (struct rpc_notify *req)
{
  struct server_handle_notify *np = req->notify;

  /* Take REQ out of the list of notified rpcs.  */
  if (req->next_req)
    req->next_req->prev_req_p = req->prev_req_p;
  *req->prev_req_p = req->next_req;

  if (np->reqs == 0)
    /* Now NP has no more reqests, so we can free it too.  */
    {
      /* Take NP out of the active list... */
      if (np->next)
	np->next->prevp = np->prevp;
      *np->prevp = np->next;

      /* And put it on the free list.  */
      np->next = _server_handle_free_server_handle_notifies;
      _server_handle_free_server_handle_notifies = np;

#warning notifications
      if (np->pending)
	/* And cancel the associated notification.  */
	{
	  server_handle_t old;
	  error_t err =
	    server_handle_request_notification (mach_task_self (), np->server_handle,
					    np->what, 0, MACH_SERVER_HANDLE_NULL,
					    MACH_MSG_TYPE_MAKE_SEND_ONCE,
					    &old);
	  if (! err && old != MACH_SERVER_HANDLE_NULL)
	    server_handle_deallocate (mach_task_self (), old);
	}	
    }
}

/* Remove RPC from the list of notified rpcs, cancelling any pending
   notifications.  _SERVER_HANDLES_LOCK should be held.  */
void
_server_handle_remove_notified_rpc (struct rpc_info *rpc)
{
  struct rpc_notify *req = rpc->notifies;

  if (req)
    /* Cancel RPC's notify requests.  */
    {
      struct rpc_notify *last = req;

      while (last->next)
	{
	  remove_req (last);
	  last = last->next;
	}
      remove_req (last);

      /* Put the whole chain on the free list.  */
      last->next = _server_handle_free_rpc_notifies;
      _server_handle_free_rpc_notifies = req;
    }
}

