/* 
   Copyright (C) 1995, 1996 Free Software Foundation, Inc.
   Written by Michael I. Bushnell.

   This file is part of the GNU Hurd.

   The GNU Hurd is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   The GNU Hurd is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "server-handles.h"
#include <assert.h>
#include <pthread.h>
#include <hurd/ihash.h>
#include <mach/notify.h>

void
server_handle_reallocate_from_external (void *server_handle_struct, server_handle_t receive)
{
  struct server_handle_info *pi = server_handle_struct;
  server_handle_status_t stat;
  int dropref = 0;
  server_handle_t foo;
  error_t err;

  err = mach_get_receive_status (mach_task_self (), receive, &stat);
  assert_perror (err);
  
  pthread_mutex_lock (&_server_handle_lock);
  
  assert (pi->server_handle_right);
  
  err = mach_mod_refs (mach_task_self (), pi->server_handle_right,
			  MACH_PORT_RIGHT_RECEIVE, -1);
  assert_perror (err);

  ihash_locp_remove (pi->bucket->htable, pi->hentry);
  
  if ((pi->flags & SERVER_HANDLE_HAS_SENDRIGHTS) && !stat.mps_srights)
    {
      dropref = 1;
      pi->flags &= ~SERVER_HANDLE_HAS_SENDRIGHTS;
    }
  else if (((pi->flags & SERVER_HANDLE_HAS_SENDRIGHTS) == 0) && stat.mps_srights)
    {
      pi->flags |= SERVER_HANDLE_HAS_SENDRIGHTS;
      pi->refcnt++;
    }
  
  pi->server_handle_right = receive;
  pi->cancel_threshold = 0;
  pi->mscount = stat.mps_mscount;
  
  ihash_add (pi->bucket->htable, receive, pi, &pi->hentry);
  pthread_mutex_unlock (&_server_handle_lock);
  
  mach_move_member (mach_task_self (), receive, pi->bucket->portset);
  
  if (stat.mps_srights)
    {
      err = server_handle_request_notification (mach_task_self (), receive,
					    MACH_NOTIFY_NO_SENDERS,
					    stat.mps_mscount, receive,
					    MACH_MSG_TYPE_MAKE_SEND_ONCE,
					    &foo);
      assert_perror (err);
      if (foo != MACH_SERVER_HANDLE_NULL)
	mach_deallocate (mach_task_self (), foo);
    }

  if (dropref)
    server_handle_server_handle_deref (pi);
}
