/* 
   Copyright (C) 1995,96, 2001,02 Free Software Foundation, Inc.
   Written by Michael I. Bushnell.

   This file is part of the GNU Hurd.

   The GNU Hurd is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   The GNU Hurd is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "server-handles.h"
#include "sh-internal.h"
#include <assert.h>
#include <pthread.h>

error_t
server_handle_reallocate (void *server_handle_struct)
{
  error_t err;
  struct server_handle_info *sh = server_handle_struct;
  int dropref = 0;

  pthread_mutex_lock (&_server_handle_lock);

  _server_handle_info_fini (sh);
  
  err = _server_handle_info_init (sh);
  if (err)
    {
      pthread_mutex_unlock (&_server_handle_lock);
      return err;
    }

  if (sh->flags & SERVER_HANDLE_HAS_USERS)
    {
      sh->flags &= ~SERVER_HANDLE_HAS_USERS;
      dropref = 1;
    }

  sh->cancel_threshold = 0;

  pthread_mutex_unlock (&_server_handle_lock);

  err = _server_handle_info_install (sh);
  if (err)
    {
      _server_handle_info_fini (sh);
      return err;
    }

  if (dropref)
    server_handle_deref (sh);

  return 0;
}

  

		      
    
