/* handle interface implementation.
   Copyright (C) 2002 Free Software Foundation, Inc.
   Written by Neal H. Walfield.

   This file is part of the GNU Hurd.

   The GNU Hurd is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   The GNU Hurd is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA. */

#include "server-handles.h"
#include <hurd/l4/types.h>

#include <stdlib.h>
#define CORBA_free free

#include "handle_server.h"

/* Interface handle */

/* XXX: Needs to be manager local.  */
extern struct server_handle_bucket *manager_bucket;

static task_t
thread_to_task (L4_ThreadId_t t)
{
  return L4_Version (t);
}

error_t
S_handle_modify_refs (CORBA_Object caller,
		      const user_handle_id_t uhid,
		      const CORBA_long delta,
		      idl4_server_environment *env)
{
  server_handle_t sh = { uhid, thread_to_task (caller) };
  struct server_handle_info *shi;
  struct user_handle_info *uhi;

  shi = server_handle_lookup (manager_bucket, sh, 0);
  if (! shi)
    return EOPNOTSUPP;

  uhi = _user_handle_info_lookup (shi, sh);
  if (! uhi)
    return EOPNOTSUPP;

  uhi->refs += delta;

  return 0;
}

error_t
S_handle_share (CORBA_Object caller,
		const user_handle_id_t uhid,
		const task_t target_task,
		idl4_server_environment *env)
{
  server_handle_t sh = { uhid, thread_to_task (caller) };
  struct server_handle_info *shi;
  struct user_handle_info *uhi;

  shi = server_handle_lookup (manager_bucket, sh, 0);
  if (! shi)
    return EOPNOTSUPP;

  uhi = _user_handle_info_lookup (shi, sh);
  if (! uhi)
    return EOPNOTSUPP;

  /* XXX: Implement me.  */
  return EOPNOTSUPP;
}

error_t
S_handle_claim (CORBA_Object caller,
		const user_handle_id_t uhid,
		idl4_server_environment *env)
{
  server_handle_t sh = { uhid, thread_to_task (caller) };
  struct server_handle_info *shi;
  struct user_handle_info *uhi;

  shi = server_handle_lookup (manager_bucket, sh, 0);
  if (! shi)
    return EOPNOTSUPP;

  uhi = _user_handle_info_lookup (shi, sh);
  if (! uhi)
    return EOPNOTSUPP;

  /* XXX: Implement me.  */
  return EOPNOTSUPP;
}

void *handle_vtable[] = HANDLE_DEFAULT_VTABLE;

void
handle_discard ()
{
}
