/* rpc interface implementation.
   Copyright (C) 2002 Free Software Foundation, Inc.
   Written by Neal H. Walfield.

   This file is part of the GNU Hurd.

   The GNU Hurd is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   The GNU Hurd is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA. */

#include <hurd/l4/types.h>

#include "server-handles.h"
#include "sh-internal.h"

#include <stdlib.h>
#define CORBA_free free

#include "rpc_server.h"

static
task_t thread_to_task (L4_ThreadId_t t)
{
  return L4_Version (t);
}

/* Cause a pending request on this object to immediately return.  The
   exact semantics are dependent on the specific object.  */
IDL4_INLINE error_t
rpc_interrupt_implementation (CORBA_Object caller,
			      const user_handle_id_t uhid,
			      idl4_server_environment *env)
{
  error_t err;
  server_handle_t sh = { uhid, thread_to_task (caller) };
  struct server_handle_info *shi;

  shi = server_handle_lookup (0, sh, 0);
  if (!shi)
    return EOPNOTSUPP;

  pthread_mutex_lock (&_server_handle_lock);

  /* This must be a valid rpc, however, more importantly, we are
     begin_rpc was not called.  */
  err = _server_handle_valid_rpc (shi, sh);
  if (err)
    {
      pthread_mutex_unlock (&_server_handle_lock);
      return EOPNOTSUPP;
    }

  if (shi->cancel_threshold < shi->seqno)
    shi->cancel_threshold = shi->seqno;
  pthread_mutex_unlock (&_server_handle_lock);

  server_handle_interrupt_rpcs (shi);
  server_handle_deref (shi);

  printf ("%s called.\n", __FUNCTION__);

  return 0;
}

void *rpc_vtable[] = RPC_DEFAULT_VTABLE;

void
rpc_discard ()
{
}
