/* 
   Copyright (C) 2002 Free Software Foundation, Inc.
   Written by Neal H. Walfield.

   This file is part of the GNU Hurd.

   The GNU Hurd is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   The GNU Hurd is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include "server-handles.h"

#include <errno.h>
#include <pthread.h>
#include <hurd/debug.h>
#include <hurd/l4/interfaces.h>
#include <l4/l4.h>
#include <idl4/platform_all.h>
#include <idl4/interface_v4.h>

#include "rpc_server.h"
#include "handle_server.h"

/* XXX: Needs to be manager local.  */
extern int (*manager_dispatcher) (int (*callback) (void *));
extern struct server_handle_bucket *manager_bucket;
extern msg_id_t manager_current_msg_id;

void
server_handle_manage_one_thread (struct server_handle_bucket *bucket,
				 server_handle_demuxer_type demuxer,
				 int timeout)
{
  error_t err;
  idl4_msgbuf_t msgbuf;
  L4_MsgTag_t msgtag;
  L4_ThreadId_t partner;
  int cnt;

  idl4_msgbuf_init (&msgbuf);

  /* We have no dispatcher as we are a single threaded server.  */
  manager_dispatcher = 0;
  manager_bucket = bucket;

  /* XXX: We need a way to either determine a reasonable default or
     provide some kind of parameter.  */
#if 0
  for (cnt = 0; cnt < THREAD_STRBUF_SIZE; cnt++)
    idl4_msgbuf_add_buffer (&msgbuf, malloc(8000), 8000);
#endif

  while (1)
    {
    restart:
      partner = IDL4_NILTHREAD;
      msgtag.raw = 0;
      cnt = 0;

      while (1)
        {
          idl4_reply_and_wait (&partner, &msgtag, &msgbuf, &cnt);
	  if (idl4_is_error (&msgtag))
	    break;

	  {
	    void cleanup (void *x)
	      {
		printf ("We (%x) were canceled\n",
			L4_Myself ().raw);
	      }

	    L4_Word_t id, interface;

	    pthread_cleanup_push (cleanup, 0);

	    err = 0;
	    interface = idl4_get_interface_id (&msgtag);
	    id = idl4_get_function_id (&msgtag);
	    manager_current_msg_id = L4_Label (msgtag);

	    switch (interface)
	      {
		extern void *rpc_vtable[];
		extern void *handle_vtable[];

	      case RPC_INTERFACE_ID:
		if (id > RPC_MAX_FID)
		  {
		    err = EOPNOTSUPP;
		    break;
		  }

		idl4_process_request (&partner, &msgtag, &msgbuf, &cnt,
				      rpc_vtable[id]);
		break;

	      case HANDLE_INTERFACE_ID:
		if (id > HANDLE_MAX_FID)
		  {
		    err = EOPNOTSUPP;
		    break;
		  }
		
		idl4_process_request (&partner, &msgtag, &msgbuf, &cnt,
				      handle_vtable[id]);
		break;

	      default:
		err = demuxer (partner, msgtag, &msgbuf, cnt);
		break;
	      }

	    pthread_cleanup_pop (0);

	    if (err)
	      goto restart;
	  }
	}
    }
}

