/* Create a user handle from a server handle.
   Copyright (C) 2002 Free Software Foundation, Inc.
   Written by Neal H. Walfield.

   This file is part of the GNU Hurd.

   The GNU Hurd is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   The GNU Hurd is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA. */

#include "server-handles.h"
#include "sh-internal.h"

#include <errno.h>
#include <stdlib.h>
#include <l4/l4.h>

error_t
_user_handle_create (struct server_handle_info *shi,
		     user_handle_t *uh,
		     task_t target_task)
{
  void install (struct user_handle_info *uhi)
    {
      uhi->task = target_task;
      uhi->refs = 1;
      uhi->next = 0;
    }

  if (! shi->installed)
    return EINVAL;

  uh->id = (L4_Word_t) shi;
  uh->server = shi->server;

  if (! shi->users.task)
    install (&shi->users);
  else
    {
      struct user_handle_info *uhi;

      for (uhi = &shi->users; uhi->next; uhi = uhi->next)
	if (uhi->task == target_task)
	  {
	    uhi->refs ++;
	    return 0;
	  }

      uhi->next = malloc (sizeof (struct user_handle_info));
      if (! uhi->next)
	return ENOMEM;

      install (uhi->next);
    }

  return 0;
}

