/* Destroy (i.e. invalidate and deallocate) a user handle.
   Copyright (C) 2002 Free Software Foundation, Inc.
   Written by Neal H. Walfield.

   This file is part of the GNU Hurd.

   The GNU Hurd is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   The GNU Hurd is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA. */

#include "server-handles.h"
#include "sh-internal.h"

#include <errno.h>
#include <stdlib.h>
#include <assert.h>

void
_user_handle_destroy (struct server_handle_info *shi,
		      server_handle_t sh)
{
  struct user_handle_info *uhi = &shi->users, *l;

  /* If there is not one entry then UH certainly does not exist.  */
  assert (uhi->task != 0);

  if (uhi->task == sh.user)
    {
      if (uhi->next)
	{
	  struct user_handle_info *foo;

	  uhi->task = uhi->next->task;
	  uhi->refs = uhi->next->refs;
	  foo = uhi->next->next;
	  free (uhi->next);
	  uhi->next = foo;
	  return;
	}
      else
	{
	  uhi->task = 0;
	  uhi->refs = 0;
	  return;
	}
    }

  for (l = uhi; uhi; l = uhi, uhi = uhi->next)
    if (uhi->task == sh.user)
      {
	l->next = uhi->next;
	free (uhi);
	return;
      }

  assert (! "UH not found!");
}
