#include <machine/vmparam.h>
#include <l4/l4.h>
#include <l4/sigma0.h>
#include <l4io.h>
#include <zalloc.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/malloc.h>

#include "task_client.h"

extern L4_Word_t _start, _end;

/* The Kernel Interface page.  */
L4_KernelInterfacePage_t *kip;

/* Sigma0's thread id.  This should be somewhere else.  */
#define sigma0_tid() (L4_GlobalId (kip->ThreadInfo.X.UserBase, 1))
/* Task-servers's thread id.  This should be somewhere else.  */
#define task_server_tid() (L4_GlobalId (kip->ThreadInfo.X.UserBase + 1, 1))

void
pager (void)
{
  int i;
  int count;
  int available = 0;

  printf ("Memory zones: {");

  for (i = ZONES - 1; i >= 0; i --)
    {
      /* printf ("Filling zone %d with pages of size %dkb\n", i,
	 1 << (i + PAGE_SHIFT - 10)); */

      count = 0;
 
      while (1)
	{
	  L4_Fpage_t block;

	  block = L4_Sigma0_GetPage (sigma0_tid (),
				     L4_FpageLog2 (-1UL << 10,
						   i + PAGE_SHIFT));

	  if (block.raw == (L4_Word_t) 0)
	    /* No more memory in this zone.  */
	    break;

	  /* Strip off the bottom bits.  */
	  block.raw &= ~0x3ff;

	  if (block.raw == (L4_Word_t) 0)
	    {
	      printf ("Mapped block at 0.\n");
	      continue;
	    }

	  zfree ((void *) block.raw, 1 << (i + PAGE_SHIFT));
	  count ++;
	}

      if (count > 0)
	printf ("%d %dkb%s", count, 1 << (i + PAGE_SHIFT - 10),
		i == 0 ? "" : ", ");
      available += ZONE_SIZE (i) * count;
    }

  printf ("}\n");
  printf ("%d MB memory available.\n", available >> 20);

  {
    int i;
    int err;
    task_t taskid;
    CORBA_Environment env;

    for (i = 0; i < 10; i ++)
      {
	env = idl4_default_environment;
	err = task_create (task_server_tid (), &taskid, &env);
	printf ("memory_server: Server returned err = %d\n");
      }
  }

  /* Do not accept any mappings or grants.  */
  L4_LoadBR (0, 0);

  printf ("Pager ready for requests; entering server loop.\n");

  /* Enter server loop.  */
  while (1)
    {
      L4_ThreadId_t tid;
      L4_MsgTag_t tag;
      L4_Msg_t msg;

      tag = L4_Wait (&tid);

      while (1)
	{
	  int i;
	  L4_Word_t mr[3];

	  /* Store the message.  */
	  L4_StoreMsg (tag, &msg);

	  for (i = 0; i < sizeof (mr) / sizeof (*mr); i ++)
	    L4_Get_Word (&msg, i, &mr[i]);

	  printf ("Pager got msg from %p, (%p, %p, %p)\n",
		  tid.raw, mr[0], mr[1], mr[2]);

	  /* The only RPC that interests us is a page fault.  */
	  if (! L4_IpcSucceeded (tag) || L4_UntypedWords (tag) != 2
	      || L4_Label (tag) != -2 || L4_TypedWords (tag) != 0)
	    {
	      printf ("malformed pagefault IPC from %p (tag=%p)\n",
		      tid.raw, tag.raw);
	      enter_kdebug ("malformed page fault");
	      break;
	    }

	  /* Clear the tag.  */
	  L4_Clear (&msg);
	  /* Get the page and give the requestor full access (i.e. rwx).  */
	  L4_Append_MapItem (&msg,
			     L4_MapItem (L4_Fpage_Set_Attrs (L4_FpageLog2 (mr[0],
									   PAGE_SHIFT),
							     L4_FullyAccessible),
					 mr[0]));

	  /* Load it and send it on its way.  */
	  L4_LoadMsg (&msg);
	  tag = L4_ReplyWait (tid, &tid);
	}
    }
}


int
main (void)
{
  L4_Fpage_t kip_area;

  kip = L4_GetKernelInterface ();
  kip_area = L4_FpageLog2 ((L4_Word_t) kip, kip->KipAreaInfo.X.s);

  printf ("Hello, world! memory-server: "
	  "tid = %d; &_start = %x; &_end = %x\n",
	  L4_Myself (), &_start, &_end);

  /* Fault in all of the pages.  */
  {
    void *p;

    for (p = &_start; (L4_Word_t) p < (L4_Word_t) &_end; p += PAGE_SIZE)
      *(volatile int *) p;
  }

  pager (); /* Never returns.  */

  return 0;
}

