#ifndef __MEMORY_H__
#define __MEMORY_H__

#include <hurd/ihash.h>

/* One for each task.  */
struct mspace
{
  posix_mutex_t lock;

  /* Statistics on memory usage.  */
  int allocated;	/* Number of allocated pages.  */
  int active;		/* Number of active pages.  */

  /* Map from handles to mobjects.  */
  ihash_t mmap;
};

typedef L4_Word_t address_t;
typedef L4_Word_t swap_t;

struct mpage; /* Forward.  */

struct mpageref
{
  posix_spinlock_t lock;

  /* The memory page.  Allocated if non-zero.  */
  struct mpage *page;

  /* List of the users of the page.  Must hold page->lock to modify
     these.  */
  struct mpageref *next;
  struct mpageref **prevp;
};

struct mpage
{
  posix_spinlock_t lock;

  int cow;		/* Needs copy on write.  */

  address_t frame;	/* If not -1, the physical frame of memory.  */
  swap_t swap_location;	/* If not -1, where the page is swapped.  */

  int refcount;		/* Number of mpageref referencing this page.  */
  struct mpageref *users; /* List of mpagerefs referencing this page.  */
};

#endif /* __MEMORY_H__ */
