#include <sys/dlmallocglue.h>
#include <machine/vmparam.h>
#include <sys/panic.h>
#include <hurd/debug.h>

#include "spin-lock.h"

/* The default size of an arena.  */
int dlmalloc_arena_size = 64 * PAGE_SIZE;

/* When to allocate a chunk rather than using an arena.  */
int dlmalloc_chunk_threshold = 64 * PAGE_SIZE;

static char malloc_data[64 * PAGE_SIZE];
static int allocated = 0;

/* Allocate memory--will always be a multiple of PAGE_SIZE.  */
void *
dlmalloc_allocate (int size)
{
  DODEBUG (3, printf ("%s (size = 0x%x)\n", __FUNCTION__,
		      size));
  if (allocated)
    return 0;

  if (size != dlmalloc_arena_size)
    DODEBUG (0, printf ("%s: Allocating %x bytes but arena size is %x.\n",
			__FUNCTION__, size, dlmalloc_arena_size));

  DODEBUG (1, printf ("%s: Allocated %x+%x\n", __FUNCTION__,
		      (unsigned int) malloc_data, size));

  allocated = 1;

  DODEBUG (3, printf ("%s returning %x\n", __FUNCTION__,
		      (unsigned int) malloc_data));

  memset (malloc_data, 0, sizeof (malloc_data));

  return malloc_data;
}

/* Deallocate memory.  */
void
dlmalloc_deallocate (void *memory, int size)
{
  if (! allocated)
    panic ("Returning unallocated data!");

  allocated = 0;
}

/* Malloc failed.  Might want to do something, e.g. panic.  */
void
dlmalloc_failed (void)
{
  panic ("dlmalloc_failed");
}

static spin_lock_t lock = SPIN_LOCK_INITIALIZER; 

int
dlmalloc_lock (void)
{
  spin_lock (&lock);
  return 0;
}

int
dlmalloc_unlock (void)
{
  spin_unlock (&lock);
  return 0;
}
