/* parse.h - Boot command lines parser.
   Copyright (C) 2002 Free Software Foundation, Inc.
   Written by Wolfgang Jhrling

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307, USA.  */

#ifndef __PARSE_H__
#define __PARSE_H__

/* A callback function is called when a $(var=func) or $(func)
   sequence is found in the input.  The callback must return a string
   allocated via malloc which will be freed automatically during
   the parser's cleanup phase.  */
struct callback
{
  char *name;
  char *(*callback) (void *handle);
};

/* A single module line, already parsed.  */
struct argv
{
  char *argz;  /* A zero-separated string list.  */
  int argc;    /* Number of strings in argz.  */
  int size;    /* Number of bytes in argz.  */
};

/* Input line.  */
struct command_line
{
  char *line;
  int length;
  void *handle;
};

/* Process all $(var=func) and $(func) statements found in
   COMMAND_LINES (which is an array of LINE_CNT elements) with allowed
   functions being CALLBACKS (an array of CALLBACK_CNT elements), and
   save the results returned by the functions in the given variable
   (it does not replace the function call with the result by rather
   with "").  In the second pass, substitute all ${var} fields with
   the values of the discovered variables.  In the third pass, parse
   the lines in a shell-like way and return the resulting argv
   vectors.  */
struct argv *parse_command_lines (struct command_line *lines, int line_cnt,
				  struct callback *callbacks, int callback_cnt);

/* Free the allocated space.  All variables are invalidated.  */
void parser_cleanup (void);

/* Add a variable with named NAME with value VALUE (which must be a
   NUL terminated string allocated via malloc) to the symbol table.
   Assumes that the variable was not previously defined.  Returns
   non-zero on failure.  VALUE will be dealloc'ed on clean up (as NAME
   is copied, it will not be).  */
int add_variable (const char *name, char *value);

/* Return the value of variable NAME.  */
char *get_variable (char *name);


#endif /* __PARSE_H__ */
